/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Input.hpp"

#include "hw/keyboard.hpp"

#include "text/text.hpp"
#include "graphics/objects.hpp"

#include GL_H

#include <math.h>
#include <string.h>

#define PADDING_FACTOR 1.8f

#define MIN_ALLOWED_CHAR 32
#define MAX_ALLOWED_CHAR 127

Input::Input() :
	MenuItem(false)
{
	clear();
}

Input::Input(float z) :
	MenuItem(false)
{
	this->mWidth = 4.0f;
	this->mHeight = 0.5f;

	this->position = Vector2(-this->mWidth / 2.0f, z);

	clear();
}

Input::~Input()
{
  // empty
}

void Input::clear()
{
	mMessage[0] = '\0';
}

String Input::getText()
{
	return mMessage;
}

void Input::update(float interval)
{
	static float time = 0.0f;

	unsigned char ch = getLastChar();

	time += interval;

	mShowCursor = time - floor(time) < 0.5;

	if (ch == KEY_ENTER || ch == KEY_ESC)
	{
		return;
	}

	if (wasKeyPressed(ch))
	{
		char *str = mMessage;
		int len = strlen(str);

		switch (ch)
		{
		case '\b':
			if (len > 0)
			{
				str[len - 1] = '\0';
			}
			break;
		default:
			if (ch >= MIN_ALLOWED_CHAR && ch <= MAX_ALLOWED_CHAR)
			{
				if (len < MAX_MESSAGE_LENGTH)
				{
					str[len] = ch;
					str[len + 1] = '\0';
				}
			}
		}
	}
}

void Input::draw() const
{
	drawPanel(mWidth, mHeight);
	drawText(mWidth, mHeight);
}

void Input::drawHUD(float width, float height) const
{
	drawRectangle(width, height);
	drawText(width, height);
}

void Input::drawText(float width, float height) const
{
	float scale = height / PADDING_FACTOR;
	float padding = (height - scale) / 2.0f;

	glColor3f(1.0f, 1.0f, 1.0f);

	glPushMatrix();
	{
		glTranslatef(padding, padding, 0.0f);

		glScalef(scale, scale, scale);

		drawStrokeThinText(mMessage);

		glTranslatef(widthStrokeText(mMessage), 0.0f, 0.0f);

		if (mShowCursor)
		{
			drawStrokeThinText("_");
		}

	}
	glPopMatrix();
}
